@@ -31,8 +31,8 @@ use crate::{
 };
 use async_trait::async_trait;
 use bytes::Bytes;
-use futures::{channel::mpsc::Sender, prelude::*};
-use std::{io::Read, sync::Arc};
+use std::sync::Arc;
+use tokio::sync::mpsc::Sender;
 
 #[derive(Debug)]
 pub struct Stat {
@@ -81,24 +81,18 @@ where
                 let user = session.user.clone();
                 let storage = Arc::clone(&session.storage);
 
-                let mut tx_success: Sender<ControlChanMsg> = args.tx_control_chan.clone();
-                let mut tx_fail: Sender<ControlChanMsg> = args.tx_control_chan.clone();
+                let tx_success: Sender<ControlChanMsg> = args.tx_control_chan.clone();
+                let tx_fail: Sender<ControlChanMsg> = args.tx_control_chan.clone();
                 let logger = args.logger;
 
                 tokio::spawn(async move {
-                    match storage.list_fmt((*user).as_ref().unwrap(), path).await {
-                        Ok(mut cursor) => {
-                            let mut result: String = String::new();
-                            match cursor.read_to_string(&mut result) {
-                                Ok(_) => {
-                                    if let Err(err) = tx_success
-                                        .send(ControlChanMsg::CommandChannelReply(Reply::new_with_string(ReplyCode::CommandOkay, result)))
-                                        .await
-                                    {
-                                        slog::warn!(logger, "{}", err);
-                                    }
-                                }
-                                Err(err) => slog::warn!(logger, "{}", err),
+                    match storage.list_vec((*user).as_ref().unwrap(), path).await {
+                        Ok(lines) => {
+                            if let Err(err) = tx_success
+                                .send(ControlChanMsg::CommandChannelReply(Reply::new_multiline(ReplyCode::CommandOkay, lines)))
+                                .await
+                            {
+                                slog::warn!(logger, "{}", err);
                             }
                         }
                         Err(e) => {
