@@ -23,9 +23,10 @@ use crate::{
     storage::{ErrorKind, Metadata, StorageBackend},
 };
 
+use crate::server::shutdown;
 use async_trait::async_trait;
 use futures_util::{SinkExt, StreamExt};
-use rustls::{ServerSession, Session as RustlsSession};
+use rustls::ServerConnection;
 use std::{net::SocketAddr, ops::Range, sync::Arc, time::Duration};
 use tokio::{
     io::{AsyncRead, AsyncWrite},
@@ -41,7 +42,7 @@ trait AsyncReadAsyncWriteSendUnpin: AsyncRead + AsyncWrite + Send + Unpin {}
 
 impl<T: AsyncRead + AsyncWrite + Send + Unpin> AsyncReadAsyncWriteSendUnpin for T {}
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Config<Storage, User>
 where
     Storage: StorageBackend<User>,
@@ -58,7 +59,7 @@ where
     pub logger: slog::Logger,
     pub ftps_required_control_chan: FtpsRequired,
     pub ftps_required_data_chan: FtpsRequired,
-    pub sitemd5: SiteMd5,
+    pub site_md5: SiteMd5,
 }
 
 /// Does TCP processing when an FTP client connects
@@ -68,6 +69,7 @@ pub async fn spawn<Storage, User>(
     tcp_stream: TcpStream,
     destination: Option<SocketAddr>,
     proxyloop_msg_tx: Option<ProxyLoopSender<Storage, User>>,
+    mut shutdown: shutdown::Listener,
 ) -> Result<(), ControlChanError>
 where
     User: UserDetail + 'static,
@@ -85,7 +87,7 @@ where
         collect_metrics,
         idle_session_timeout,
         logger,
-        sitemd5,
+        site_md5: sitemd5,
         ..
     } = config;
 
@@ -173,17 +175,22 @@ where
                             true => incoming = None,
                             false => incoming = Some(Err(ControlChanError::new(ControlChanErrorKind::ControlChannelTimeout)))
                         };
+                    },
+                    _ = shutdown.listen() => {
+                        slog::info!(logger, "Shutting down control loop");
+                        incoming = Some(Ok(Event::InternalMsg(ControlChanMsg::Quit)))
+                        // TODO: Do we want to wait a bit for a data transfer to complete i.e. session.data_busy is true?
                     }
                 };
                 incoming
             };
             match incoming {
-                None => {}
+                None => {} // Loop again
+                Some(Ok(Event::InternalMsg(ControlChanMsg::Quit))) => {
+                    slog::info!(logger, "Exiting control loop");
+                    return;
+                }
                 Some(Ok(event)) => {
-                    if let Event::InternalMsg(ControlChanMsg::Quit) = event {
-                        return;
-                    }
-
                     if let Event::InternalMsg(ControlChanMsg::SecureControlChannel) = event {
                         slog::info!(logger, "Upgrading control channel to TLS");
 
@@ -199,8 +206,8 @@ where
                         let accepted = acceptor.accept(io).await;
                         let io: Box<dyn AsyncReadAsyncWriteSendUnpin> = match accepted {
                             Ok(stream) => {
-                                let s: &ServerSession = stream.get_ref().1;
-                                if let Some(certs) = s.get_peer_certificates() {
+                                let s: &ServerConnection = stream.get_ref().1;
+                                if let Some(certs) = s.peer_certificates() {
                                     let mut session = shared_session.lock().await;
                                     session.cert_chain = Some(certs.iter().map(|c| crate::auth::ClientCert(c.0.clone())).collect());
                                 }
